@@ -72,6 +72,7 @@ public interface DriveAction<T extends DriveVersion> extends Comparable<DriveAct
     static final String PARAMETER_QUARANTINE = "quarantine";
     static final String PARAMETER_RESET = "reset";
     static final String PARAMETER_LENGTH = "length";
+    static final String PARAMETER_STOP = "stop";
 
     static final String PARAMETER_DIRECT_LINK = "directLink";
     static final String PARAMETER_DIRECT_LINK_FRAGMENTS = "directLinkFragments";
@@ -80,7 +81,7 @@ public interface DriveAction<T extends DriveVersion> extends Comparable<DriveAct
 
     static final Set<String> PARAMETER_NAMES = Collections.unmodifiableSet(new HashSet<String>(Arrays.asList(new String[] {
         PARAMETER_PATH, PARAMETER_TOTAL_LENGTH, PARAMETER_OFFSET, PARAMETER_CONTENT_TYPE, PARAMETER_ERROR, PARAMETER_QUARANTINE,
-        PARAMETER_MODIFIED, PARAMETER_CREATED, PARAMETER_RESET, PARAMETER_LENGTH,
+        PARAMETER_MODIFIED, PARAMETER_CREATED, PARAMETER_RESET, PARAMETER_LENGTH, PARAMETER_STOP,
         PARAMETER_DIRECT_LINK, PARAMETER_DIRECT_LINK_FRAGMENTS, PARAMETER_PREVIEW_LINK, PARAMETER_THUMBNAIL_LINK
     })));
 
